/*  BOiNG Arcade Game
    2020 - Gord Payne
    Sponsored by Elegoo
*/

#include <LiquidCrystal.h>// library for 16x02 LCD Display
// create the LCD display object
LiquidCrystal lcd(7, 6, 5, 4, 3, 2);

#include "pitches.h" // Note values
// look at pitches.h header file
int speakerPin = 8; // passive piezo buzzer pin

#include "LedControl.h" // library for MAX7219 8x8 matrix display
// create the 8x8 LED matrix display object
LedControl lc = LedControl(12, 10, 11, 1);


int ballX;// ball x and y coordinates (starts at x = 1, y = 3)
int ballY;
int xM;// x and y direction multipliers - reverse when hit wall or paddle
int yM;
int p1y, p2y; // player paddle y cooridnates
int p1pin = A1;// analog pins for player 1 and player 2 paddles (10K potentiometers)
int p2pin = A2;
int p1Score;// player scores
int p2Score;
int maxScore = 5;// score for end of game - adjust as you like

long ballTime;
long bThresh = 50;// pause for ball redraw
boolean bon;
boolean ballReturn;
int lastP1 = 0;// last direction of paddle, for direction of return
int lastP2 = 0;
void setup() {
  Serial.begin(9600);// open path for Serial Monitor - optional for diagnostics
  lc.shutdown(0, false);// wake up 8x8 matrix display
  /* Set the brightness to a medium values */
  lc.setIntensity(0, 0); // was 0,8     0,0 is dimmest. 0,1 good for normal use
  /* and clear the display */
  lc.clearDisplay(0);
  pinMode(p1pin, INPUT);// define the potentiometers as input
  pinMode(p2pin, INPUT);
  pinMode(speakerPin, OUTPUT);
  lcd.begin(16, 2);// start the LCD display
  // LCD Coordinates - order is (column,row)
  // where column is 0 to 15, row is 0 or 1
  splashScreen();

  int p1Val = analogRead(A1);// read player 1 paddle value
  p1y = int(map(p1Val, 0, 1023, 1, 7));//scale pot value to value between 1 and 7
  int p2Val = analogRead(A2);
  p2y = int( map(p2Val, 0, 1023, 1, 7)); //scale pot value to value between 1 and 7
  drawPaddle(0, p1y);// draw current paddle positions
  drawPaddle(7, p2y);
  newGame();
  ballTime = millis();
  bon = false;
  ballReturn = false;
}

void loop() {
  if ((p1Score < maxScore) && (p2Score < maxScore)) {// if no winner yet

    int p1Val = analogRead(A1);// read player 1 paddle value
    p1y = int(map(p1Val, 0, 1023, 1, 7));//scale pot value to value between 1 and 7
    int p2Val = analogRead(A2);
    p2y = int( map(p2Val, 0, 1023, 1, 7)); //scale pot value to value between 1 and 7
    drawPaddle(0, p1y);// draw current paddle positions
    drawPaddle(7, p2y);
    if ((millis() - ballTime > bThresh) && bon == false) {
      ballOn();
      bon = true;
      ballTime = millis();
    }
    if ((millis() - ballTime > bThresh) && bon == true) {
      ballOff();
      bon = false;
      ballTime = millis();
    }

    if (ballX == 7 && (ballY == p2y || ballY == p2y - 1 || ballY == p2y + 1)) { // check for ball return
      if (p2y > lastP2) {// change player 2 return y direction
        yM = 1;
      } else {
       yM = -1;
      }
      lastP2 = p2y;
      ballReturn = true;
      xM = xM * -1;// reverse ball direction back to other player
      ballX = ballX + xM;
      tone(speakerPin, NOTE_C4, 200);// play paddle hit sound
    } else if (ballX == 0 && (ballY == p1y || ballY == p1y - 1 || ballY == p1y + 1)) {
      if (p1y > lastP1) { // change player 1 return y diretion
        yM = 1;
      } else {
        yM = -1;
      }
      lastP1 = p1y;
      ballReturn = true;
      xM = xM * -1;// reverse ball direction back to other player
      ballX = ballX + xM;
      tone(speakerPin, NOTE_C4, 200);// play paddle hit sound
    } else if (ballX == 7 && ((ballY > p2y + 1) || (ballY < p2y - 1))) { // player 1 scores
      tone(speakerPin, NOTE_C2, 400);
      p1Score ++;// add to player 1 score
      // display new score
      lcd.setCursor(2, 1);
      lcd.print(p1Score);
      ballX = 1;// reset ball coordinates for next serve
      xM = 1;
      ballY = p1y;
      delay(400);
      lc.clearDisplay(0);
      reDrawPaddles();
      delay(100);
    } else if (ballX == 0 && ((ballY > p1y + 1) || (ballY < p1y - 1))) { // player 2 scores
      tone(speakerPin, NOTE_C2, 400);
      p2Score  ++;
      lcd.setCursor(12, 1);
      lcd.print(p2Score);
      ballX = 6;
      xM = -1;
      ballY = p2y;
      delay(400);
      lc.clearDisplay(0);
      reDrawPaddles();
      delay(100);
    }

  } else { // somebody won the game
    doWin();// display win message on LCD
    delay(2000);
    newGame();
    delay(1000);
  }
  delay(20);// slow loop a bit
}

void ballOn() {
  if (ballY == 7 || ballY == 0) {// if at top or bottom of display
    yM = yM * -1;// reflect ball off wall
    tone(speakerPin, NOTE_E4, 20);// play ball-bounce sound
  }
  if (ballReturn == false) {// if not a ball return
    ballX = ballX + xM;// update ball position
  } else {
    ballReturn = false;// was a return, reset flag
  }
  ballY = ballY + yM;
  lc.setLed(0, ballX, ballY, true);

}
void ballOff() {
  lc.setLed(0, ballX, ballY, false);
}



void drawPaddle(int x, int y) {// update paddle position
  lc.setLed(0, x, y - 2, false);// erase outside LED before redraw
  lc.setLed(0, x, y - 1, true);
  lc.setLed(0, x, y, true);
  lc.setLed(0, x, y + 1, true);
  lc.setLed(0, x, y + 2, false);//erase outside LED before redraw
}

void doWin() {// check for a player win

  tone(speakerPin, NOTE_C4, 600);// play victory sound
  delay(300);
  tone(speakerPin, NOTE_C5, 600);
  delay(300);
  tone(speakerPin, NOTE_C6, 600);
  delay(300);
  lcd.setCursor(1, 1);
  if (p1Score == 5) {//give winner message
    lcd.print("Player 1 WINS!");
  } else {
    lcd.print("Player 2 WINS!");
  }

}

void newGame() {// reset scores and start new game
  lc.clearDisplay(0);
  drawPaddle(0, p1y);// draw current paddle positions
  drawPaddle(7, p2y);
  p1Score = 0;
  p2Score = 0;
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Plyr 1");
  lcd.setCursor(10, 0);
  lcd.print("Plyr 2");
  lcd.setCursor(2, 1);
  lcd.print(p1Score);
  lcd.setCursor(12, 1);
  lcd.print(p2Score);
  if (p1y > p2y) {
    ballX = 1;
    ballY = p1y;// starting y-coordinate for ball
    xM = 1;
    yM = 1;
  } else {
    ballX = 6;
    ballY = p2y;
    xM = -1;
    yM = -1;
  }
}

void splashScreen() {
  tone(speakerPin, NOTE_C6, 600);
  lcd.setCursor(5, 0);
  lcd.print("POiNG!");
  tone(speakerPin, NOTE_C4, 600);
  delay(500);//noTone(speakerPin);
  lcd.setCursor(1, 1);
  lcd.print("2020-Gord Payne");
  tone(speakerPin, NOTE_C5, 600);
  delay(1000);
  lcd.clear();
  lcd.setCursor(2, 0);
  lcd.print("Sponsored by");
  lcd.setCursor(5, 1);
  lcd.print("Elegoo");
  tone(speakerPin, NOTE_C6, 600);
  delay(800);
  lcd.clear();
}

void reDrawPaddles() {
  drawPaddle(0, p1y);// draw current paddle positions
  drawPaddle(7, p2y);
}
